if (typeof Array.prototype.forEach != "function") {
	Array.prototype.forEach = function (fn, context) {
		for (var k = 0, length = this.length; k < length; k++) {
			if (typeof fn === "function" && Object.prototype.hasOwnProperty.call(this, k)) {
				fn.call(context, this[k], k, this);
			}
		}
	};
}

if (typeof Array.prototype.map != "function") {
	Array.prototype.map = function (fn, context) {
		var arr = [];
		if (typeof fn === "function") {
			for (var k = 0, length = this.length; k < length; k++) {
				arr.push(fn.call(context, this[k], k, this));
			}
		}
		return arr;
	};
}

if (typeof Array.prototype.filter != "function") {
	Array.prototype.filter = function (fn, context) {
		var arr = [];
		if (typeof fn === "function") {
			for (var k = 0, length = this.length; k < length; k++) {
				fn.call(context, this[k], k, this) && arr.push(this[k]);
			}
		}
		return arr;
	};
}

if (typeof Array.prototype.some != "function") {
	Array.prototype.some = function (fn, context) {
		var passed = false;
		if (typeof fn === "function") {
			for (var k = 0, length = this.length; k < length; k++) {
				if (passed === true) break;
				passed = !!fn.call(context, this[k], k, this);
			}
		}
		return passed;
	};
}

if (typeof Array.prototype.every != "function") {
	Array.prototype.every = function (fn, context) {
		var passed = true;
		if (typeof fn === "function") {
			for (var k = 0, length = this.length; k < length; k++) {
				if (passed === false) break;
				passed = !!fn.call(context, this[k], k, this);
			}
		}
		return passed;
	};
}


var pharosComm = angular.module("pharosComm", []);

pharosComm.config(function($translateProvider){
	/* sanitize tranlation */
	$translateProvider.useSanitizeValueStrategy('escape');

	/* load language file */
	$translateProvider.useStaticFilesLoader({
    	prefix: "./locale/",
        suffix: ".json"
    });

	/* default language */
	$translateProvider.preferredLanguage("en_US");
});

pharosComm.config(["$httpProvider", function($httpProvider){
	$httpProvider.defaults.headers.post["Content-Type"] = "application/json;charset=utf-8";
	// $httpProvider.defaults.headers.post["Content-Type"] = "application/x-www-form-urlencoded;charset=utf-8";

	// var param = function(obj){
	// 	var query = '';
	// 	var name;
	// 	var value;
	// 	for(name in obj){
	// 		value = obj[name];
	// 		if (value instanceof Array ){
	// 			for (var i = 0; i < value.length; i++) {
	// 				query += encodeURIComponent(name) + '=' + encodeURIComponent(value[i]) + '&';
	// 			};
	// 		} else if (value instanceof Object) {
	// 			query += encodeURIComponent(name) + '=' + encodeURIComponent(angular.toJson(value)) + '&';
	// 		} else if (value !== undefined && value !== null) {
	// 			query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
	// 		}
	// 	}
	// 	return query.length ? query.substr(0, query.length - 1): query;
	// };

	// $httpProvider.defaults.transformRequest = function(data){
	// 	return angular.isObject(data) && String(data) != '[object File]' ? param(data) : data;
	// };
}]);

pharosComm.factory('$proxy', ["$http", "$user", "$rootScope", function($http, $user, $rootScope){
	var defaults = {
		type: "POST",
		url: null,
		timeout: 15*1000
	}

	var credentialId = $user.getCookie("NAME") ? $user.getCookie("NAME").split("&amp")[2] : "";
	return {
		request: function(options, data, success, failure, error){
			var proxyConfig = angular.extend(defaults, options);
			// var credentialId = "";
			data = angular.extend({CREDENTIAL: credentialId}, data);

			success = success || function(){};
			failure = failure || function(){};
			error = error || function(){};

			$http({
				method: proxyConfig.type,
				url: proxyConfig.url,
				data: proxyConfig.type === "POST" ? data : "",
				params: proxyConfig.type === "GET" ? data : "",
				timeout: proxyConfig.timeout
			}).success(function(data, status, headers, config){
				if(data.RESPONSE != "ERROR"){
					success(data.DATA, data.other);
				}else{
					failure(data.ERR_MSG, data);
				}
			}).error(function(data, status, headers, config){
				error(status);
				switch(status){
					case 0:
					case 401:
						$rootScope.$broadcast("Unauthorized401");
						// location.href = "./login.html";
				}
			});
		}
	};
}]);

pharosComm.factory("$user", ["$document",function($document){
	var loginUser = {
	  username: "",
	  role: ""
	};
	return {
		setCookie: function(data){
			document.cookie = "NAME=" + data.name + "&amp" + data.role + "&amp" + data.credential;
			// document.cookie = "NAME=" + data.name;
			// document.cookie = "ROLE=" + data.role;
			// document.cookie = "CREDENTIAL="+ data.credential;
		},
		delCookie: function(name){
			var _data = new Date();
			document.cookie = name + "=a;expires=" + _data.toGMTString();
		},
		getCookie: function(c_name){
			var _cookie = document.cookie;
			if(_cookie.length > 0){
				var _start = _cookie.indexOf(c_name + "=");
				var _end;
				if(_start !== -1){
					_start = _start + c_name.length + 1;
					_end = _cookie.indexOf(";", _start);
					if(_end == -1) _end = _cookie.length;
					return _cookie.substring(_start, _end);
				}
			}
			return "";
		},
		checkCookie: function(){
			try{
				// localStorage.setItem();
				if(!navigator.cookieEnabled){
					return false;
				}
			}catch(e){
				return false;
			}
			return true;
		}
	}
}]);

pharosComm.factory("$form", ["$proxy", "$vtype", "$tool", "$msg", "$encrypt", function($proxy, $vtype, $tool, $msg, $encrypt){
	var defaults = {
		proxy: null,
		fields: null,
		success: null,
		failure: null,
		error: null,
		validator: null,
		autoLoad: true,
		autoInit: true,
		prompt: {
			show: true,
			success: "FORM.SUCCESS",
			failure: "FORM.FAILED"
		}
	}
	// var disabled = false;
	return {
		config: function(scope, options){
			var form = this;

			scope.form = {};
			scope.form.proxy = angular.extend({}, defaults.proxy, options.proxy);
			scope.form.fields = angular.extend([], defaults.fields, options.fields);
			scope.form.serials = {};
			scope.form.success = options.success || function(){};
			scope.form.failure = options.failure || function(){};
			scope.form.error = options.error || function(){};
			scope.form.validator = options.validator || function(scope){return true};
			scope.form.autoLoad = angular.isDefined(options.autoLoad) ? options.autoLoad : defaults.autoLoad;
			scope.form.autoInit = angular.isDefined(options.autoInit) ? options.autoInit : defaults.autoInit;
			scope.form.prompt = angular.extend({}, defaults.prompt, options.prompt);

			scope.form.disabled = false;

			if(scope.form.autoLoad){
				form.init(scope);
				form.load(scope);
			}else if(scope.form.autoInit){
				form.init(scope);
			}
		},
		init: function(scope){
			var form = this;
			scope.isForm = true;
			scope.form = scope.form || {};

			var fields = scope.form.fields;
			var data = {};
			var snapshot = [];

			for(var index = 0, len = fields.length; index < len; index++){
				var name = fields[index].name;
				// fields[index].mapping = fields[index].mapping || name;

				data[name] = {};
				data[name].data = "";
				// data[name].mapping = fields[index].mapping;

				data[name].disabled = fields[index].disabled || false;
				data[name].readOnly = fields[index].readOnly || false;
				data[name].autoTrim = angular.isDefined(fields[index].autoTrim) ? fields[index].autoTrim : true;
				data[name].visible = angular.isDefined(fields[index].visible) ? fields[index].visible : true;

				data[name].valid = angular.isDefined(fields[index].valid) ? fields[index].valid : true;

				data[name].validate = fields[index].validate || null;
				data[name].vtype = fields[index].vtype || null;
				data[name].format = fields[index].format || null;
				data[name].allowBlank = angular.isDefined(fields[index].allowBlank) ? fields[index].allowBlank : true;
				data[name].allowEmptyObject = angular.isDefined(fields[index].allowEmptyObject) ? fields[index].allowEmptyObject : true;
				data[name].maxLength = fields[index].maxLength || -1;
				data[name].minLength = fields[index].minLength || -1;

				var encryption = fields[index].encryption;
				data[name].encryption = ( angular.isDefined( encryption ) || encryption === null ) ? encryption : undefined;

				/*data[name].setNormal = (function(name){
					return function(){
						data[name].valid = true;
					}
				})(name);
				data[name].setError = (function(name){
					return function(){
						data[name].valid = false;
					}
				})(name);*/

				snapshot[name] = "";
			}

			scope.form.data = angular.extend(data, scope.form.data);
			// remember original data
			scope.form.snapshot = snapshot;
			scope.form.passSnapshot = [];
		},
		loadData: function(scope, data){
			if(scope.isForm){
				var fields = scope.form.fields;
				var snapshot = {};
				var passSnapshot = {};

				for(var index = 0,len = fields.length; index < len; index++){
					var name = fields[index].name;
					var mapping = fields[index].mapping || name;

					scope.form.data[name].data = angular.isDefined(data[mapping])? data[mapping] : "";
					snapshot[name] = angular.isDefined(data[mapping])? data[mapping] : "";

					// original data, there may has a bug
					if(scope.form.data[name].encryption){
						passSnapshot[name] = scope.form.data[name].inputData;
					}

					// form need to reset
				}
				scope.form.snapshot = snapshot;
				scope.form.passSnapshot = passSnapshot;
			}
		},
		load: function(scope, data, success, failure, error){
			if(scope.isForm){
				var form = this;
				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};

				$proxy.request(scope.form.proxy, data, function(data, other){
					form.loadData(scope, data);
					success(data, other);
				}, function(errMsg,data){
					failure(errMsg, data);
				}, function(status){
					error(status);
				});
			}
		},
		setNormal: function(scope){
			if(scope.isForm){
				var fields = scope.form.fields;
				fields.forEach(function(value, index){
					scope.form.data[value.name].valid = true;
				});
			}
		},
		validate: function(scope){
			if(scope.isForm){
				this.setNormal(scope);
				var fields = scope.form.fields;
				var serials = scope.form.serials;
				for(var index = 0,  len = fields.length; index < len; index++){
					var name = fields[index].name;
					var mapping = fields[index].mapping || name;

					if(!serials[name] && !scope.form.data[name].disabled){
						var val = "";
						if(scope.form.data[name].encryption || scope.form.data[name].encryption === null){
							val = scope.form.data[name].inputData || "";
						}else{
							val = scope.form.data[name].data;
						}

						if(scope.form.data[name].allowEmptyObject === false){
							if(val === null || angular.equals({}, val)){
								scope.form.data[name].valid = false;
								return false;
							}
						}

						// deal with allowBlank
						if( scope.form.data[name].allowBlank === false){
							if(val === ""){
								// error deal
								scope.form.data[name].valid = false;
								return false;
							}
						}else{
							if(val === ""){
								continue;
							}
						}

						//  deal with minLength
						if(scope.form.data[name].minLength !== -1){
							if(angular.isString(val) && val.length < scope.form.data[name].minLength){
								// deal with error
								scope.form.data[name].valid = false;
								return false;
							}
						}

						// deal with maxLength, use maxLength property
						if(scope.form.data[name].maxLength !== -1){
							if(angular.isString(val) && val.length > scope.form.data[name].maxLength){
								scope.form.data[name].valid = false;
								return false;
							}
						}

						var vtype = scope.form.data[name].vtype;
						if(vtype !== null){
							if(angular.isString(vtype)){
								if($vtype[vtype](val) !== true){
									// deal with error
									scope.form.data[name].valid = false;
									return false;
								}
							}else if(angular.isObject(vtype)){
								if($vtype[vtype.vtype](val, vtype.config) !== true){
									scope.form.data[name].valid = false;
									return false;
								}
							}
						}

						if(angular.isFunction(scope.form.data[name].validate)){
							if(scope.form.data[name].validate() === false){
								scope.form.data[name].valid = false;
								return false;
							}
						}
					}
				}

				if(!scope.form.validator(scope)){
					return false;
				};

				return true;
			}
		},
		submit: function(scope, data, success, failure, error, showPrompt, loadData, doValidate, clearData){
			if(scope.isForm){
				var form = this;
				if(scope.form.disabled){
					return;
				}
				if(doValidate !== false && !form.validate(scope)){
					return false;
				}

				var formData = {};
				var fields = scope.form.fields;
				var serials = scope.form.serials;
				for(var index = 0, len = fields.length; index < len; index++){
					var name = fields[index].name;
					var mapping = fields[index].mapping || name;

					if(!serials[name] && !scope.form.data[name].disabled){
						if(angular.isObject(scope.form.data[name].encryption)){
							var encrypt = $encrypt[scope.form.data[name].encryption.name];
							var encryptParam = scope.form.data[name].encryption.params;

							// the method of encryption is undefined

							scope.form.data[name].data = encrypt ? encrypt(scope.form.data[name].inputData, encryptParam) : scope.form.data[name].inputData;
						}else if(scope.form.data[name].encryption === null){
							scope.form.data[name].data = scope.form.data[name].inputData;
						}
						formData[mapping] = angular.isDefined(scope.form.data[name].data) ? scope.form.data[name].data : null;
					}
				}

				var temp = data.DATA ? angular.extend({}, formData, data.DATA) : formData;
				formData = $tool.isEmptyObject(temp) ? angular.extend({}, data) : angular.extend({}, data, {"DATA": temp}) ;

				scope.form.serials = {};

				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};

				showPrompt = angular.isDefined(showPrompt) ? showPrompt : true;

				if(scope.form.submitBtn == undefined){
					scope.form.submitBtn = {};
				}
				scope.form.submitBtn.disabled = true;

				if(scope.form.prompt.show && showPrompt){
					// create msg
					var successPrompt = $msg.create({
						type: "success",
						text: scope.form.prompt.success
					});

					var failurePrompt = $msg.create({
						type: "failure",
						text: scope.form.prompt.failure
					});
				}

				$proxy.request(scope.form.proxy, formData, function(data, other){
					if(data && loadData == true){
						form.loadData(scope, data);
					}
					success(data, other);
					// deal with success
					if(clearData !== false){
						form.clear(scope, data);
					}
					scope.form.submitBtn.disabled = false;
					if(showPrompt){
						// $rootScope.promptMsg.content = "FORM.SUCCESS";
						// $rootScope.promptMsg.show = true;
						successPrompt.show();
					}
				}, function(errMsg, data){
					failure(errMsg, data);
					// deal with failure
					if(clearData !== false){
						form.clear(scope, data);
					}
					scope.form.submitBtn.disabled = false;
					if(showPrompt){
						// $rootScope.promptMsg.content = "FORM.FAILED";
						// $rootScope.promptMsg.show = true;
						failurePrompt.show();
					}
				}, function(status){
					error(status);
					// deal with error
					if(clearData !== false){
						form.clear(scope, data);
					}
					scope.form.submitBtn.disabled = false;
					if(showPrompt){
						// $rootScope.promptMsg.content = "FORM.FAILED";
						// $rootScope.promptMsg.show = true;
						failurePrompt.show();
					}
				});
			}
		},
		serialize: function(scope, data){
			for(var index = 0, len = data.length; index < len; index++){
				scope.form.serials[data[index]] = 1;
			}
		},
		clear: function(scope){
			if(scope.isForm){
				var fields = scope.form.fields;
				for(var index = 0, len = fields.length; index < len; index++){
					var name = fields[index].name;
					var mapping = fields[index].mapping || name;

					scope.form.data[name].data = "";
					scope.form.data[name].valid = true;
					if ( scope.form.data[name].inputData ){
						scope.form.data[name].inputData = "";
					}
				}
			}
		}
	}
}]);

pharosComm.factory("$grid", ["$form", "$proxy", function($form, $proxy){
	var defaults = {
		proxy: null,
		fields: null,
		// columns: null,

		autoLoad: true,
		autoInit: true,
		maxRulesProperty: "max_rules"
	};

	var updateGrid = function(grid,data){
		var fields = grid.fields;
		if(data){
			for(var row = 0, len = data.length; row < len; row++){
				var item = {};
				for(var index = 0, fieldLen = fields.length; index < fieldLen; index++){
					var name = fields[index].name;
					var mapping = fields[index].mapping || name;

					item[name] = {};
					item[name].data = angular.isDefined(data[row][mapping]) ? data[row][mapping] : "";
				}
				item.key = data[row][grid.key];
				grid.data.push(item);
				// grid.keyToIndex.push(item.key);
				grid.keyToIndex[row] = item.key;
			}
		}
	}

	return {
		config: function(scope, options){
			scope.grid = {};
			scope.grid.key = options.key?options.key+"ID":"";
			scope.grid.proxy = angular.extend({}, defaults.proxy, options.proxy);
			scope.grid.fields = angular.extend([], defaults.fields, options.fields);
			// scope.grid.columns = angular.extend([], defaults.columns, options.columns);

			scope.grid.autoLoad = angular.isDefined(options.autoLoad) ? options.autoLoad : defaults.autoLoad;
			scope.grid.autoInit = angular.isDefined(options.autoInit) ? options.autoInit : defaults.autoInit;
			scope.grid.maxRulesProperty = options.maxRulesProperty || defaults.maxRulesProperty;
			scope.grid.keyToIndex = [];

			scope.grid.selectedIndexs = [];
			scope.grid.isEditing = false;

			scope.grid.data = [];

			if(scope.grid.autoLoad){
				this.init(scope);
				this.load(scope);
			} else if(scope.grid.autoInit){
				this.init(scope);
			}

			if(angular.isObject(options.editor)){
				var params = {};
				params.proxy = angular.extend({}, defaults.proxy, options.proxy, options.editor.proxy);
				params.fields = options.editor.fields ? angular.extend([], defaults.fields, options.editor.fields) : angular.extend([], defaults.fields, options.fields);
				params.success = options.editor.success || options.success || function(){};
				params.failure = options.editor.failure || options.failure || function(){};
				params.error = options.editor.error || options.error || function(){};
				params.autoLoad = false;
				params.autoInit = options.editor.autoInit || options.autoInit || defaults.autoInit;
				params.validator = options.editor.validator || null;

				$form.config(scope, params);
			};
		},
		init: function(scope){
			scope.isGrid = true;
			var $grid = this;

			scope.grid.loadData = function(data, operation, isForm, clearSelected){
				$grid.loadData(scope, data, operation, isForm, clearSelected);
			};
			scope.grid.load = function(data, success, failure, error){
				$grid.load(scope, data, success, failure, error);
			};
			scope.grid.insert = function(data, success, failure, error, loadData){
				$grid.insert(scope, data, success, failure, error, loadData);
			};
			scope.grid.update = function(data, success, failure, error, formScope, loadData){
				$grid.update(scope, data, success, failure, error, formScope, loadData);
			};
			scope.grid.remove = function(data, success, failure, error){
				$grid.remove(scope, data, success, failure, error);
			};
			scope.grid.startEdit = function(index, msgFlag, form){
				$grid.startEdit(scope, index, msgFlag, form);
			};
			scope.grid.cancelEdit = function(){
				$grid.cancelEdit(scope);
			};
			scope.grid.startAdd = function(msgFlag){
				$grid.startAdd(scope, msgFlag);
			};
			scope.grid.startRemove = function(delItems, msgFlag){
				$grid.startRemove(scope, delItems, msgFlag);
			}
		},
		loadData: function(scope, data, operation , isForm, clearSelected){
			// all response data is the grid data
			if(scope.isGrid && angular.isString(scope.grid.key)){
				var fields = scope.grid.fields;
				if(operation === "load"){
					scope.grid.data = [];
					if(clearSelected !== false){
						scope.grid.selectedIndexs = [];
					}
					updateGrid(scope.grid, data);
					/*if(data){
						for(var row = 0; row < data.length; row++){
							var item = {};
							for(var index = 0; index < fields.length; index++){
								var name = fields[index].name;
								var mapping = fields[index].mapping || name;

								item[name] = {};
								item[name].data = angular.isDefined(data[row][mapping]) ? data[row][mapping] : "";
								// add other property
							}
							item.key = data[row][scope.grid.key];
							scope.grid.data.push(item);
						}
					}*/
				}else if(operation === "insert"){
					updateGrid(scope.grid, data);
					/*if(data){
						var item = {};
						for(var index = 0; index < fields.length; index++){
							var name = fields[index].name;
							var mapping = fields[index].mapping || name;

							item[name] = {};
							item[name].data = angular.isDefined(data[mapping]) ? data[mapping] : "";
						}
						item.key = data[scope.grid.key];
						scope.grid.data.push(item);
					}*/
				}else if(operation === "update"){
					if(data){
						var updateIndex = scope.grid.updateIndex ||  scope.grid.selectedIndexs[0];
						if(isForm === true){
							var serials = data.serials;
							var fields = data.fields;

							for(var index = 0, len = fields.length; index < len; index++){
								var name = fields[index].name;
								selectedIndexs = scope.grid.selectedIndexs;
								if(scope.grid.data[updateIndex][name] && angular.isDefined(data.data[name].data)){
									if(!serials[name] && !data.data[name].disabled){
										for(var sIndex = 0, selectedLen = selectedIndexs.length; sIndex < selectedLen; sIndex++){
											var selectedIndex = selectedIndexs[sIndex];
											scope.grid.data[selectedIndex][name].data = data.data[name].data;
										}
									}
								}
							}
						}else{
							// var updateIndex = scope.grid.updateIndex;
							for(var index = 0; index < fields.length; index++){
								var name = fields[index].name;
								var mapping = fields[index].mapping || name;

								angular.isDefined(data[mapping]) ? scope.grid.data[updateIndex][name].data = data[mapping] : null;
							}
						}
					}
				}else if(operation === "remove"){
					if(data){
						var deleteItems = [];
						for(var index = 0; index < data.length; index++){
							// deleteItems.push(data[index][scope.grid.key]-1);
							var deleteItem = scope.grid.keyToIndex.indexOf(data[index][scope.grid.key]);
							if(deleteItem !== -1){
								deleteItems.push(deleteItem);
							}
						}
						deleteItems.sort(function(preVal, nextVal){
							return nextVal - preVal;
						});
						for(var len = deleteItems.length, index = 0; index < len; index++){
							scope.grid.data.splice(deleteItems[index], 1);
							scope.grid.keyToIndex.splice(deleteItems[index], 1);
						}
					}
				}
			}
		},
		load: function(scope, data, success, failure, error){
			var grid = this;
			if(scope.isGrid){
				var gridname = angular.uppercase(scope.grid.key.replace("ID", ""));
				var data = angular.extend({
					REQUEST: "GET_"+ gridname + "_LIST"
				}, data);

				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};

				$proxy.request(scope.grid.proxy, data, function(res, other){
					var data = res;
					if(angular.isObject(res) && !angular.isArray(res)){
						data = res["data"] || [];
					}
					grid.loadData(scope, data, "load");
					success(res, other);
				}, function(errMsg, res){
					failure(errMsg, res);
				}, function(status){
					error(status);
				});
			}
		},
		insert: function(scope, data, success, failure, error, loadData){
			if(scope.isGrid && scope.grid.proxy.url != null){
				var grid = this;

				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};
				var gridname = angular.uppercase(scope.grid.key.replace("ID", ""));
				data = angular.extend(data, {"REQUEST": "ADD_" + gridname});

				if(scope.isForm){
					var showPrompt = undefined;
					if(loadData && loadData.showPrompt === false){
						showPrompt = false;
					}
					var clearData = undefined;
					if(loadData && loadData.clearData === false){
						clearData = false;
					}
					// scope, data, success, failure, error, showPrompt, loadData, doValidate, clearData
					$form.submit(scope, data, function(data,other){
						var formData = angular.copy(data);
						loadData === false ? null : grid.loadData(scope, formData, "load");
						success(data, other);
						// scope.grid.isEditing = false;
						// $form.clear(scope);
					}, function(errMsg){
						failure(errMsg, data);
					}, function(status){
						error(status);
					}, showPrompt, undefined, undefined, clearData);
				}
			}
		},
		update: function(scope, data, success, failure, error, formScope, loadData){
			var showPrompt = undefined;
			if(loadData && loadData.showPrompt === false){
				showPrompt = false;
			}
			var clearData = undefined;
			if(loadData && loadData.clearData === false){
				clearData = false;
			}

			formScope = angular.isDefined(formScope) ? formScope : null;
			if(scope.isGrid && !formScope && scope.grid.proxy.url != null){
				var grid = this;
				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};

				var gridname = angular.uppercase(scope.grid.key.replace("ID", ""));
				var tmp = {"REQUEST": "EDIT_" + gridname};

				// tmp[scope.grid.key] = scope.grid.data[scope.grid.updateIndex].key;
				idData = {};
				idData[scope.grid.key] = scope.grid.data[scope.grid.updateIndex].key;

				data = angular.extend({}, tmp, data);
				data["DATA"] = angular.extend({}, idData, data["DATA"]);

				if(scope.isForm){

					// scope, data, success, failure, error, showPrompt, loadData, doValidate, clearData
					$form.submit(scope, data, function(data, other){
						var formData = angular.copy(data);

						loadData === false ? null : grid.loadData(scope, formData, "load");
						success(data, other);
						scope.grid.updateIndex = "";
						// scope.grid.isEditing = false;
						// $form.clear(scope);
					}, function(errMsg, data){
						failure(errMsg, data);
					}, function(status){
						error(status);
					}, showPrompt, undefined, undefined, clearData);
				}
			}else if(scope.isGrid && formScope){
				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};

				$form.submit(formScope, data, function(data, other){
					loadData === false ? null : scope.grid.loadData(formScope.form, "update", true);
					success(data, other);
				}, function(errMsg, data){
					failure(errMsg, data);
				},function(status){
					error(status);
				}, showPrompt, undefined, undefined, clearData);
			}
		},
		remove: function(scope, data, success, failure, error){
			if(scope.isGrid && scope.grid.proxy.url != null){
				var grid = this;
				var success = success || function(){};
				var failure = failure || function(){};
				var error = error || function(){};

				var gridname = angular.uppercase(scope.grid.key.replace("ID", ""));
				var tmp = {"REQUEST": "DELETE_" + gridname};
				data = angular.extend({}, data, tmp);

				$proxy.request(scope.grid.proxy, data, function(data, other){
					grid.loadData(scope, data, "load");
					success(data, other);
				}, function(errMsg, data){
					failure(errMsg, data);
				}, function(status){
					error(status);
				});
			}
		},
		startEdit: function(scope, index , msgFlag, form){
			if(scope.isGrid && scope.isForm){
				scope.grid.updateIndex = index;
				msgFlag = angular.isDefined(msgFlag) ? msgFlag : true;
				if(msgFlag || angular.isDefined(form)){
					var name, fields = scope.form.fields;
					var formData = scope.form.data;
					if(angular.isDefined(form)){
						fields = form.fields;
						formData = form.data;
					}
					for(var index = 0, len = fields.length; index < len; index++){
						name = fields[index].name;
						if(scope.grid.data[scope.grid.updateIndex][name] && angular.isDefined(scope.grid.data[scope.grid.updateIndex][name].data)){
							formData[name].data = scope.grid.data[scope.grid.updateIndex][name].data;
						}else{
							formData[name].data = "";
						}
					}

					// device 的 edit 有三个按钮，在此处黑一下
					var _data = { type: "edit" };
					if ( scope.editDevicAddonBtn ) {
						_data.addonBtn = scope.editDevicAddonBtn;
					}

					if(angular.isObject(msgFlag)){
						_data.options = msgFlag;
						return scope.$broadcast("gridEditing", _data);
					}

					msgFlag === false ? null : scope.$broadcast("gridEditing", _data);
				}
				// scope.grid.isEditing = true;
			}
		},
		cancelEdit: function(scope){
			if(scope.isGrid){
				if(scope.grid.updateIndex !== undefined){
					scope.grid.updateIndex = "";
				}
				$form.clear(scope);
				// scope.grid.isEditing = false;
			}
		},
		startAdd: function(scope, msgFlag){
			if(scope.isGrid){
				scope.grid.updateIndex = "";

				if(angular.isObject(msgFlag)){
					return scope.$broadcast("gridEditing", {type:"add", options: msgFlag});
				}
				msgFlag === false ? null : scope.$broadcast("gridEditing", {type:"add"});
				// scope.grid.isEditing = true;
			}
		},
		startRemove: function(scope, delItems, msgFlag){
			if(scope.isGrid){
				if(angular.isObject(msgFlag)){
					return scope.$broadcast("gridEditing", {type:"remove", "items": delItems, options: msgFlag});
				}
				msgFlag === false ? null : scope.$broadcast("gridEditing", {type: "remove", "items": delItems});
			}
		}
	}
}]);

pharosComm.factory("$msg", ["$timeout", "$tool", function($timeout, $tool){
	var statusMessages = [];

	var statusDefaults = {
		type: "",
		text: "",
		timeout: 0,
		autoDestory: true
	}

	function objIndexOf(obj, arr){
		for(var index = 0, len = arr.length; index < len; index++){
			if(arr[index].id == obj.id){
				return index;
			}
		}
		return -1;
	}

	function Message(options){
		$msg = this;

		switch(options.type){
			case "success":
			case "failure":
				angular.extend(this, statusDefaults, options);
				this.timeout = options.timeout ? options.timeout : 1000;
				this.id = "sm" + $tool.randomId();
				this.mode = "status";
				break;
			default:

		}
	}

	Message.prototype = {
		constructor: Message,
		show: function(){
			var $msg = this;

			// if($msg.showFlag === true){
			// 	return;
			// }

			if($msg.mode == "status"){
				var index = objIndexOf($msg, statusMessages);
				if(index == -1){
					statusMessages.push($msg);
				}
			}

			if($msg.timeout != 0){
				$timeout(function(){
					$msg.close();
				}, $msg.timeout);
			}
		},
		close: function(){
			var $msg = this;

			if($msg.mode == "status"){
				var index = objIndexOf($msg, statusMessages);
				if(index !== -1){
					statusMessages.splice(index, 1);
				}
			}
		}
	};

	return {
		statusMessages: statusMessages,
		create: function(options){
			var newMsg = new Message(options);

			return newMsg;
		}
	}

}]);

pharosComm.factory("$url", function(){
	return {
		format: function(oldUrl){
			var subs = "";
			return subs + oldUrl;
		}
	}
});

pharosComm.factory("$vtype", ["$translate", function($translate){
	var _type = {
		"phone_number": function(value){
			var defaults = {};
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000001"
			};
			var pattern = {
				regex: /^\+?[0-9]\d*$/
			};
			var options = angular.extend({}, defaults, arguments[1]);

			if(pattern.regex.test(value)){
				return true;
			}else{
				return errorcode.vtypeText;
			}
		},
		"accountname": function(value){
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000013"
			}
			var pattern = {
				regex: /^[A-Za-z0-9]+$/
			}
			if(!pattern.regex.test(value)) return errorcode.vtypeText;
			return true;
		},
		"ip": function(value){
			var defaults = {
				allowAllZeroFlag: false,
				allowLoopFlag: false,
				allowDTypeFlag: false,
				allowETypeFlag: false,
				allowAllOneFlag: false
			};
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000001",
				disallowAllZeroText: "ERROR.e000002",
				disallowLoopText: "ERROR.e000003",
				disallowDTypeText: "ERROR.e000004",
				disallowETypeText: "ERROR.e000005",
				disallowAllOneText: "ERROR.e000006",
				disallowFirstZeroText: "ERROR.e000007",
				disallowFirstAllOneText: "ERROR.e000008"
			};
			var pattern = {
					regex: /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])(\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])){3}$/
				}
				//var vtypeText = " {{'ERROR_IP' | translate}}";


			var options = angular.extend({}, defaults, arguments[1]);

			if (pattern.regex.test(value)) {
				ipStr = value;
				var szarray = [0, 0, 0, 0];
				var remain;
				var i;
				for (i = 0; i < 3; i++) {
					var n = ipStr.indexOf('.');
					szarray[i] = ipStr.substring(0, n);
					remain = ipStr.substring(n + 1);
					ipStr = remain;
				}
				szarray[3] = remain;
				for (i = 0; i < 4; i++) {
					if (szarray[i] < 0 || szarray[i] > 255) {
						return errorcode.vtypeText;

					}
				}
				if (!options.allowLoopFlag) {
					if (szarray[0] == 127) {
						return errorcode.disallowLoopText;
						//return false;
					}
				}
				if (!options.allowDTypeFlag) {
					if (szarray[0] >= 224 && szarray[0] <= 239) {
						//return false;
						return errorcode.disallowDTypeText;
					}
				}
				if (!options.allowETypeFlag) {
					if (szarray[0] >= 240 && szarray[0] <= 254) {
						//return false;
						return errorcode.disallowETypeText;
					}
				}
				if (!options.allowAllOneFlag) {
					// if(szarray[0]==255)
					// {
					// 	return this.disallowAllOneText;
					// }
					if (szarray[0] == 255 && szarray[1] == 255 && szarray[2] == 255 && szarray[3] == 255) {
						//return false;
						return errorcode.disallowAllOneText;
					}
					if(szarray[0]==255)
					{
						return errorcode.disallowFirstAllOneText;
					}
				} else {
					if (szarray[0] == 255 && szarray[1] == 255 && szarray[2] == 255 && szarray[3] == 255) {} else {
						if (szarray[0] == 255) {
							//return false;
							return errorcode.disallowFirstAllOneText;
						}
					}
				}
				if (!options.allowAllZeroFlag) {
					if (szarray[0] == 0 && szarray[1] == 0 && szarray[2] == 0 && szarray[3] == 0) {
						//return false;
						return errorcode.disallowAllZeroText;
					}
				}
				if (!options.allowAllZeroFlag && !options.allowFirstZeroFlag) {
					if (szarray[0] == 0) {
						//return false;
						return errorcode.disallowFirstZeroText;
					}
				}
				return true;
			} else {
				return errorcode.vtypeText;
			}
		},
		"mac": function(value){
			var defaults = {
				allowMultiFlag: true
			};
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000001",
				disallowAllMultiText: "ERROR.e000001"
			};
			var pattern = {
				regex: /^[a-fA-F\d]{2}\-[a-fA-F\d]{2}\-[a-fA-F\d]{2}\-[a-fA-F\d]{2}\-[a-fA-F\d]{2}\-[a-fA-F\d]{2}$/,
				patternMulti: /^\s*[0-9A-Fa-f]{1}[13579bdfBDF]{1}(\-[A-Fa-f0-9]{2}){5}\s*$/,
				patternZero: /^(0{1,2}-){5}0{1,2}$/
			};

			var options = angular.extend({}, defaults, arguments[1]);
			if (pattern.regex.test(value)) {
				if (options.allowMultiFlag) {
					if (pattern.patternMulti.test(value)) {
						return errorcode.disallowAllMultiText;
					}
				}
				if (pattern.patternZero.test(value)) {
					return errorcode.vtypeText;
				}
				return true;
			} else {
				return errorcode.vtypeText;
			}
		},
		"ip_netmask": function(value){
			var defaults = {};
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000014",
				MASK_TOO_SHORT: "ERROR.e000012"
			};
			var pattern = {
				regex: /^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\/([0-2]?\d|3[0-2])$/
			}

			if(pattern.regex.test(value)){
				var mask = value.slice(-2);
				if(mask.charAt(0) == '/' || mask < 21){
					return errorcode.MASK_TOO_SHORT;
				}
			}else{
				return errorcode.vtypeText;
			}

			return true;
		},
		"float_number": function(value){
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000001",
				NUMBER_MIN: "ERROR.e000009",
				NUMBER_MAX: "ERROR.e000010",
				NUMBER_MIN_MAX: "ERROR.e000011",
				AND: "ERROR.AND"
			};
			var defaults = {};
			var pattern = {
				regex: /^-?\d+(.\d{1,})?$/
			}
			var options = angular.extend({}, defaults, arguments[1]);

			if(pattern.regex.test(value)){
				value = angular.isString(value) ? parseFloat(value) : value;
				if(options.max !== null && options.max !== undefined){
					if(value > options.max){
						if(options.min !== null && options.min !== undefined){
							var str = $translate.instant(errorcode.NUMBER_MIN_MAX) + options.min.toString() + $translate.instant(errorcode.AND) + options.max.toString();
							return str;
						}else{
							var str = $translate.instant(errorcode.NUMBER_MAX) + options.max.toString();
							return str;
						}
					}
				}

				if(options.min !== null && options.min !== undefined){
					if(value < options.min){
						if(options.max !== null && options.max !== undefined){
							var str = $translate.instant(errorcode.NUMBER_MIN_MAX) + options.min.toString() + $translate.instant(errorcode.AND) + options.max.toString();
							return str;
						}else{
							var str = $translate.instant(errorcode.NUMBER_MIN) + options.min.toString();
							return str;
						}
					}
				}
				return true;
			}else{
				return errorcode.vtypeText;
			}
		},
		"number": function(value){
			var defaults = {};
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000001",
				NUMBER_MIN: "ERROR.e000009",
				NUMBER_MAX: "ERROR.e000010",
				NUMBER_MIN_MAX: "ERROR.e000011",
				AND: "ERROR.AND"
			};
			var pattern = {
				regex: /^-?[0-9]\d*$/
			};
			var options = angular.extend({}, defaults, arguments[1]);

			if(pattern.regex.test(value)){
				value = parseInt(value, 10);
				if(options.max !== null && options.max !== undefined ){
					if(value > options.max){
						if(options.min !== null && options.min !== undefined){
							var str = $translate.instant(errorcode.NUMBER_MIN_MAX) + options.min.toString() + $translate.instant(errorcode.AND) + options.max.toString();
							return str;
						}else{
							var str = $translate.instant(errorcode.NUMBER_MAX) + options.max.toString();
							return str;
						}
					}
				}

				if(options.min !== null && options.min !== undefined){
					if(value < options.min){
						if(options.max !== null && options.max !== undefined){
							var str = $translate.instant(errorcode.NUMBER_MIN_MAX) + options.min.toString() + $translate.instant(errorcode.AND) + options.max.toString();
							return str;
						}else{
							var str = $translate.instant(errorcode.NUMBER_MIN) + options.min.toString();
							return str;
						}
					}
				}

				return true;
			}else{
				return errorcode.vtypeText;
			}
		},
		"email": function(value){
			var errorcode = {
				valid: "ERROR.e000000",
				vtypeText: "ERROR.e000001"
			}
			var pattern = {
				// regex: /^(\w+\.)*\w+@(\w+\.)+[A-Za-z]+$/
				regex: /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/
			}
			if( !pattern.regex.test(value) ) return errorcode.vtypeText;
			return true;
		}
	}

	return _type;
}]);

pharosComm.factory("$encrypt", function(){
	// encryption type
	var _md5 = function(val){
		return val;
	}

	var _noop = function(val){
		return val;
	}

	var _sha1 = function(val){
		var hexcase = 0;
		var chrsz = 8;
		function hex_sha1(s) {
		    return binb2hex(core_sha1(str2binb(s), s.length * chrsz));
		}
		function core_sha1(x, len) {
		    x[len >> 5] |= 0x80 << (24 - len % 32);
		    x[((len + 64 >> 9) << 4) + 15] = len;

		    var w = Array(80);
		    var a = 1732584193;
		    var b = -271733879;
		    var c = -1732584194;
		    var d = 271733878;
		    var e = -1009589776;

		    for (var i = 0; i < x.length; i += 16) {
		        var olda = a;
		        var oldb = b;
		        var oldc = c;
		        var oldd = d;
		        var olde = e;

		        for (var j = 0; j < 80; j++) {
		            if (j < 16) w[j] = x[i + j];
		            else w[j] = rol(w[j - 3] ^ w[j - 8] ^ w[j - 14] ^ w[j - 16], 1);
		            var t = safe_add(safe_add(rol(a, 5), sha1_ft(j, b, c, d)), safe_add(safe_add(e, w[j]), sha1_kt(j)));
		            e = d;
		            d = c;
		            c = rol(b, 30);
		            b = a;
		            a = t;
		        }

		        a = safe_add(a, olda);
		        b = safe_add(b, oldb);
		        c = safe_add(c, oldc);
		        d = safe_add(d, oldd);
		        e = safe_add(e, olde);
		    }
		    return Array(a, b, c, d, e);
		}
		function sha1_ft(t, b, c, d) {
		    if (t < 20) return (b & c) | ((~b) & d);
		    if (t < 40) return b ^ c ^ d;
		    if (t < 60) return (b & c) | (b & d) | (c & d);
		    return b ^ c ^ d;
		}
		function sha1_kt(t) {
		    return (t < 20) ? 1518500249 : (t < 40) ? 1859775393 : (t < 60) ? -1894007588 : -899497514;
		}
		function safe_add(x, y) {
		    var lsw = (x & 0xFFFF) + (y & 0xFFFF);
		    var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
		    return (msw << 16) | (lsw & 0xFFFF);
		}
		function rol(num, cnt) {
		    return (num << cnt) | (num >>> (32 - cnt));
		}
		function str2binb(str) {
		    var bin = Array();
		    var mask = (1 << chrsz) - 1;
		    for (var i = 0; i < str.length * chrsz; i += chrsz)
		    bin[i >> 5] |= (str.charCodeAt(i / chrsz) & mask) << (24 - i % 32);
		    return bin;
		}
		function binb2hex(binarray) {
		    var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
		    var str = "";
		    for (var i = 0; i < binarray.length * 4; i++) {
		        str += hex_tab.charAt((binarray[i >> 2] >> ((3 - i % 4) * 8 + 4)) & 0xF) + hex_tab.charAt((binarray[i >> 2] >> ((3 - i % 4) * 8)) & 0xF);
		    }
		    return str;
		}
		return hex_sha1(val);
	}

	return {
		md5: _md5,
		hex_sha1: _sha1,
		noop: _noop
	}
});

/*pharosComm.factory("$format", function(){
	_format: {
		"floatFormat": function(val){
			if(!isNaN(parseFloat(val))){
				return val.toFixed(1);
			}else{
				return val;
			}
		}
	}
	return _format;
})*/

pharosComm.factory("$tool", function($translate){
	function uuid() {
	    var s = [];
	    var hexDigits = "0123456789abcdef";
	    for (var i = 0; i < 36; i++) {
	        s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
	    }
	    s[14] = "4";  // bits 12-15 of the time_hi_and_version field to 0010
	    s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1);  // bits 6-7 of the clock_seq_hi_and_reserved to 01
	    s[8] = s[13] = s[18] = s[23] = "-";

	    var uuid = s.join("");
	    return uuid;
	}

	function isEmptyObject(obj){
		for(var index in obj){
			return false;
		}
		return true;
	}

	function timeFormat(data, format){
		if(data){
			var t = new Date(data);
			format = angular.isDefined(format) ? format : $translate.instant("TASK.TIMEFORMAT");

			var tf = function(i){
				return (i < 10 ? '0' : '') + i;
			};

			return format.replace(/yyyy|MM|dd|hh|mm|ss/g, function(a){
				var result = "";
				switch(a){
					case "yyyy":
						result = tf(t.getFullYear());
						break;
					case "MM":
						result = tf(t.getMonth() + 1);
						break;
					case "dd":
						result = tf(t.getDate());
						break;
					case "hh":
						result = tf(t.getHours());
						break;
					case "mm":
						result = tf(t.getMinutes());
						break;
					case "ss":
						result = tf(t.getSeconds());
						break;
				}
				return result;
			});
		}
	}

	function prefixDateStr(str) {
		return str > 9
			? str
			: "0" + str;
	}

	function timeFormat2(date){
		var time = new Date(date.time);

		var year = time.getFullYear();
		var month = time.getMonth() + 1;
		var day = time.getDate();
		month = prefixDateStr(month);
		day = prefixDateStr(day);

		var hour = time.getHours();
		var minute = time.getMinutes();
		hour = prefixDateStr(hour);
		minute = prefixDateStr(minute);

        return day + "/" + month + "/" + year + " " + hour + ":" + minute;
	}

	function expressFilter(condition, fromGrid){
		return function(item){
			if(angular.isArray(condition)){
				for(var index = 0, len = condition.length; index < len; index++){
					if(condition[index] == item.deviceID){
						return true;
					}
				}
			}else if(angular.isObject(condition)){
				if(filter(condition)){
					return true;
				}
			}else{
				return true;
			}

			function filter(expression){
				if(expression.type === "complex"){
					if(expression.operator === "&&"){
						return filter(expression.condition[0]) && filter(expression.condition[1]);
					}else if(expression.operator === "||"){
						return filter(expression.condition[0]) || filter(expression.condition[1]);
					}
				}else if(expression.type === "simple"){
					var itemData = item[expression.field];
					switch(expression.comparator){
						case "contains":
							return itemData.indexOf(expression.value) == -1 ? false: true;
						case "startswith":
							return itemData.indexOf(expression.value) == 0 ? true: false;
						case "=":
							return itemData === expression.value;
						case "!=":
							return itemData !== expression.value;
						case ">":
							return itemData > expression.value;
						case "<":
							return itemData < expression.value;
					}
				}
			}
		};
	}

	function randomId(){
		return "id" + (new Date()).getTime() + parseInt(Math.random()*100000);
	}

	function uniqueArr(array){
		array.sort(function compare(value1, value2) {
				return value1 - value2;
		});

		for (i = array.length - 1; i > 0; i--){
				if (array[i] === array[i-1]){
						array.splice(i, 1);
				}
		}
	}

	// 若 array1 中的元素在 array2 中存在，删除它
	function orthogonalize(array1, array2){
			var i, ilen = array1.length;
			var j, jlen = array2.length;
			for (i = ilen - 1; i > -1; i--){
				for(j = 0; j < jlen; j++){
					 if( array1[i] === array2[j] ){
						 array1.splice(i, 1);
					 }
				}
			}
	}

	return {
		createUuid: uuid,
		isEmptyObject: isEmptyObject,
		timeFormat: timeFormat,
		timeFormat2: timeFormat2,
		expressFilter: expressFilter,
		randomId: randomId,
		uniqueArr: uniqueArr,
		orthogonalize: orthogonalize
	}
});

pharosModule.factory("$constant", function(){
	return{
		COUNTRY:{
			"511": "REGION.TESTMODE"
		},
		WIRELESSTYPE:{
			WIRELESS_TYPE_2_4G: 1,
			WIRELESS_TYPE_5G: 2
		},
		MODE:{
			"MODE_B": 	"DEVICE.MODE_B", 	// 802.11b
			"MODE_G": 	"DEVICE.MODE_G",	// 802.11g
			"MODE_N": 	"DEVICE.MODE_N",	// 802.11n
			"MODE_BG": 	"DEVICE.MODE_BG",	// 802.11b/g
			"MODE_BGN": "DEVICE.MODE_BGN",	// 802.11b/g/n
			"MODE_A": 	"DEVICE.MODE_A",	// 802.11a
			"MODE_AN": 	"DEVICE.MODE_AN"	// 802.11a/n
		},
		MXTXRATE: {
			"B": [
				["0.3 Mbps", "0.5 Mbps", "1.4 Mbps", "2.8 Mbps"],  // "5MHz":
				["0.5 Mbps", "1 Mbps", "2.8 Mbps", "5.5 Mbps"],  // "10MHz":
				["1 Mbps", "2 Mbps", "5.5 Mbps", "11 Mbps"]  // "20MHz":
			],
			"A_BG_G": [
				["1.5 Mbps", "2.3 Mbps", "3 Mbps", "4.5 Mbps", "6 Mbps", "9 Mbps", "12 Mbps", "13.5 Mbps"],  // "5MHz":
				["3 Mbps", "4.5 Mbps", "6 Mbps", "9 Mbps", "12 Mbps", "18 Mbps", "24 Mbps", "27 Mbps"],  // "10MHz":
				["6 Mbps", "9 Mbps", "12 Mbps", "18 Mbps", "24 Mbps", "36 Mbps", "48 Mbps", "54 Mbps"],  // "20MHz":
				["6 Mbps", "9 Mbps", "12 Mbps", "18 Mbps", "24 Mbps", "36 Mbps", "48 Mbps", "54 Mbps"]  // "20_40MHz":
			],
			"N_TKIP": [
				["1.5 Mbps", "2.3 Mbps", "3 Mbps", "4.5 Mbps", "6 Mbps", "9 Mbps", "12 Mbps", "13.5 Mbps"],  // "5MHz":
				["3 Mbps", "4.5 Mbps", "6 Mbps", "9 Mbps", "12 Mbps", "18 Mbps", "24 Mbps", "27 Mbps"],  // "10MHz":
				["6 Mbps", "9 Mbps", "12 Mbps", "18 Mbps", "24 Mbps", "36 Mbps", "48 Mbps", "54 Mbps"],  // "20MHz":
				["6 Mbps", "9 Mbps", "12 Mbps", "18 Mbps", "24 Mbps", "36 Mbps", "48 Mbps", "54 Mbps"]  // "20_40MHz":
			],
			"N": [
				[
					"MCS0 - 1.6/1.8 Mbps", "MCS1 - 3.3/3.6 Mbps", "MCS2 - 4.9/5.4 Mbps", "MCS3 - 6.5/7.2 Mbps",
					"MCS4 - 9.8/10.8 Mbps", "MCS5 - 13/14.4 Mbps", "MCS6 - 14.6/16.3 Mbps", "MCS7 - 16.3/18.1 Mbps",
					"MCS12 - 19.5/21.7 Mbps", "MCS13 - 26/28.9 Mbps", "MCS14 - 29.3/32.5 Mbps", "MCS15 - 32.5/36.1 Mbps"
				],  // "5MHz":
				[
					"MCS0 - 3.3/3.6 Mbps", "MCS1 - 6.5/7.2 Mbps", "MCS2 - 9.8/10.8 Mbps", "MCS3 - 13/14.4 Mbps",
					"MCS4 - 19.5/21.7 Mbps", "MCS5 - 26/28.9 Mbps", "MCS6 - 29.3/32.5 Mbps", "MCS7 - 32.5/36.1 Mbps",
					"MCS12 - 39/43.3 Mbps", "MCS13 - 52/57.8 Mbps", "MCS14 - 58.5/65 Mbps", "MCS15 - 65/72.2 Mbps"
				],  // "10MHz":
				[
					"MCS0 - 6.5/7.2 Mbps", "MCS1 - 13/14.4 Mbps", "MCS2 - 19.5/21.7 Mbps", "MCS3 - 26/28.9 Mbps",
					"MCS4 - 39/43.3 Mbps", "MCS5 - 52/57.8 Mbps", "MCS6 - 58.5/65 Mbps", "MCS7 - 65/72.2 Mbps",
					"MCS12 - 78/86.7 Mbps", "MCS13 - 104/115.6 Mbps", "MCS14 - 117/130 Mbps", "MCS15 - 130/144.4 Mbps"
				],  // "20MHz":
				[
					"MCS0 - 13.5/15 Mbps", "MCS1 - 27/30 Mbps", "MCS2 - 40.5/45 Mbps", "MCS3 - 54/60 Mbps",
					"MCS4 - 81/90 Mbps", "MCS5 - 108/120 Mbps", "MCS6 - 121.5/135 Mbps", "MCS7 - 135/150 Mbps",
					"MCS12 - 162/180 Mbps", "MCS13 - 216/240 Mbps", "MCS14 - 243/270 Mbps", "MCS15 - 270/300 Mbps"
				]  // "20_40MHz":
			]
		},
		WIRELESS_MODE: ["Auto", "Access Point", "Client", "Repeater", "Bridge", "AP Router", "AP Client Router"]
	}
});

pharosModule.factory("$recurseTree", ['$compile', function($compile) {
	return {
        compile: function(element, link){
            if(angular.isFunction(link)){
                link = { post: link };
            }
            var contents = element.contents().remove();
            var compiledContents;
            return {
                pre: (link && link.pre) ? link.pre : null,
                post: function(scope, element){
                    if(!compiledContents){
                        compiledContents = $compile(contents);
                    }
                    compiledContents(scope, function(clone){
                        element.append(clone);
                    });
                    if(link && link.post){
                        link.post.apply(null, arguments);
                    }
                }
            };
        }
	}
}]);

pharosModule.filter("integer", function(){
	return function(val){
		if(isNaN(Number(val))){
			return val;
		}else{
			return Math.round(val);
		}
	}
});

/*pharosModule.filter("", function(){
	return function(condition){

	}
});*/


pharosModule.factory("$tree", ['$proxy', '$url', function($proxy, $url) {
	return{
		editTree: function(name, data, success, failure, error){
			$proxy.request({
				url: $url.format("/" + name + "/")
			},{
				"REQUEST": "EDIT_" + angular.uppercase(name) + "_TREE",
				"DATA": data
			}, success, failure, error);
		},
		getTree: function(name, success, failure, error){
			name = (name == "device") ? "device_group" : name;
			$proxy.request({
				url: $url.format("/" + name + "/"),
			},{
				"REQUEST": "GET_" + angular.uppercase(name) + "_TREE"
			}, success, failure, error);
		}
	}
}]);
pharosModule.factory("$infoComm", ['$tree', '$proxy', '$url', '$grid',
function($tree, $proxy, $url, $grid) {
	function getSlideGroup( scope, name ){
		var	data = getMenuData(scope, name),
			groupItems = [];
		if ( data === false ) return false;
		traverseTree( data, function(item){
			groupItems.push( {name: item.name} );
		} )
		groupItems.shift();
		return groupItems;
	}
	function traverseTree(tree, fn){
		var queue = [],
			item,
			children;

		queue.push( tree );
		while( queue.length ){
			item = queue.shift();
			angular.isFunction(fn) && fn(item);
			children = item.children;
			if ( children && children.length ){
				for ( var i = 0, len = children.length; i< len; i++ ){
					queue.push( children[i] );
				}
			}
		}
	}
	function getNodeByName( tree, nodeName ){
		var result = false;
		traverseTree( tree, function(item){
			if ( item.name == nodeName ){
				result = item;
			}
		} )
		return result;
	}
	function getMenuData(scope, name){
		var _data = scope[name + "Menu"].data;
			if ( _data && _data[name + "Tree"] ){
				return _data[name + "Tree"];
			}
		return false;
	}
	function getGridData( scope, name ){
		var _data = scope.grid.data;
		if (_data) {
			return _data;
		};
		return false;
	}
	function compare( value1, value2 ){
		return value1 - value2;
	}
	// function removeSame(listArr, listObj){
	function removeSame(name, item, listObj){
		var listArr = item[ name + "IDList" ];
		var ismodified = false;
		if( !angular.isArray(listArr) || listArr.length == 0 || !angular.isObject(listObj) ) return false;
		for( i = listArr.length - 1; i > -1; i-- ){
			if ( listObj[listArr[i]] ) {
				listArr.splice( i, 1 );
				ismodified = true;
			}
		}
		if ( ismodified ) return true;
		return false;
	}

	return {
		updateSlideGroup: function(scope, name){
			scope.information.groupItems = getSlideGroup( scope, name );
			if (scope.toolbarConfig){
				var i, len;
				var show = (name == "manual" || name=="firmware") ? scope.toolbarConfig.operator.more : scope.toolbarConfig.operator.show;
				for ( i = 0, len = show.length; i < len; i++ ){
					if ( show[i].slider_holder == "grouping" ){
						show[i].slide = scope.information.groupItems;
					}
				}
			}
		},
		updateRootIDList: function(scope, name, data){
			if( !data || data.length == 0 ) return false;
			var _data = scope[name + "Menu"].data;
			if ( _data && _data[name + "Tree"] ){
				_data[name + "Tree"][name + "IDList"] = [];
				for ( var i = 0, len = data.length; i < len; i++ ){
					_data[name + "Tree"][name + "IDList"].push( data[i][name + "ID"].data );
				}
			}
		},
		deleteIDList: function(scope, name, idList, success, fail, error){
			var i, len, _idList = {}, modifiedArr = [];
			var menuData  = getMenuData( scope, name ),
				_menuData = angular.copy( menuData );
			var _name = name;
			if ( name == "manual" ) {
					name = "device";
			}
			if ( !idList || idList.length == 0 ) return false;
			idList.sort(compare);
			for ( i = 0, len = idList.length; i < len; i++ ){
				_idList[idList[i]] = true;
			}

			// traverseTree( menuData, function(item){
			// 	var result = removeSame( name, item, _idList);
			// 	if (result) modifiedArr.push( { uuid: item.uuid} );
			// } )

			var failure = function(){
				menuData = _menuData;
				angular.isFunction(fail) && fail();
			}
			var data = {};
			data[ name + "IDList" ] = idList;
			$proxy.request({
				url: $url.format("/" + name + "/"),
			},{
				"REQUEST": "DELETE_" + angular.uppercase(name),
				"DATA": data
			}, function(data){
				$grid.loadData( scope, data, "load" );
				// var editTreeData = {};
				// editTreeData[_name+"Tree"] = scope[_name + "Menu"].data[_name + "Tree"];
				// editTreeData["modified"] = modifiedArr;
				// $tree.editTree(name, editTreeData, success, failure, error);

				traverseTree( menuData, function(item){
					var result = removeSame( name, item, _idList);
					// if (result) modifiedArr.push( { uuid: item.uuid} );
				} )
				angular.isFunction(success) && success();
			}, failure, error);
		},
		activateIDList: function(scope, name, idList, success, fail, error){
			var data = {};
			data[ name + "IDList" ] = idList;
			$proxy.request({
				url: $url.format("/" + name + "/"),
			},{
				"REQUEST": "ACTIVATE_" + angular.uppercase(name),
				"DATA": data
			}, success, fail, error);
		},
		deactivateIDList: function(scope, name, idList, success, fail, error){
			var data = {};
			data[ name + "IDList" ] = idList;
			$proxy.request({
				url: $url.format("/" + name + "/"),
			},{
				"REQUEST": "DEACTIVATE_" + angular.uppercase(name),
				"DATA": data
			}, success, fail, error);
		},
		removeFromGroup: function(scope, name, success, fail, error){
			var IDList = {},
				index, cur, _cur,
				_data, modified = [];
			for ( var i = 0, len = scope.grid.selectedIndexs.length; i < len; i++ ){
				index = scope.grid.selectedIndexs[i];
				IDList[scope.grid.data[index].key] = true;
			}
			cur = scope.information.curFolder[name + "IDList"];
			_cur = angular.copy(cur);
			if (cur) {
				for( i = cur.length - 1; i > -1; i-- ){
					if ( IDList[cur[i]] ) cur.splice(i,1);
				}
				modified.push( scope.information.curFolder );
			};
			var failure = function(){
				scope.information.curFolder[name + "IDList"] = _cur;
				angular.isFunction(fail) && failure();
			}

			var _name = name;
			if ( name =="device" ) _name = "manual";

			_data = angular.copy( scope[_name + "Menu"].data );
			_data.modified = modified;
			$tree.editTree(name, _data, success, failure, error);
		},
		addToGroup: function(scope, name, groupObj, success, fail, error){
			var IDList = {}, _IDList = [],
				index, cur, _cur, curNode, _data,
				modified = [];
			for ( var i = 0, len = scope.grid.selectedIndexs.length; i < len; i++ ){
				index = scope.grid.selectedIndexs[i];
				IDList[scope.grid.data[index].key] = true;
				_IDList.push(scope.grid.data[index].key);
			}

			// deal with device's scope
			var _name = name;
			if ( name == "device") _name = "manual";
			_data = scope[_name + "Menu"].data[_name + "Tree"];

			curNode = getNodeByName( _data, groupObj.name );
			if ( !curNode ) return false;
			cur = curNode[name + "IDList"];
			_cur = angular.copy(cur);
			if (cur) {
				for( i = cur.length - 1; i > -1 ; i-- ){
					if ( IDList[cur[i]] ) cur.splice(i,1);
				}
				cur = cur.concat( _IDList );
				curNode[name + "IDList"] = cur;
				modified.push( curNode );
			};
			var failure = function(){
				curNode[name + "IDList"] = _cur;
				angular.isFunction(fail) && failure();
			}
			_data = angular.copy( scope[_name + "Menu"].data );
			_data.modified = modified;
			$tree.editTree(name, _data, success, fail, error);
		}
	}
}]);

pharosModule.factory("$toolBarSrv", [ function(){
	return {
		setEnable: function( toolbarConfig ){
			var i = 0,
					len = toolbarConfig.operator.show.length;
			for ( ; i < len; i++ ){
				toolbarConfig.operator.show[i].isDisabled = "enable";
			}
			if ( toolbarConfig.operator.more ){
				toolbarConfig.operator.more.isDisabled = "enable";
			}
		},
		setDisable: function( toolbarConfig ){
			var i = 0,
			len = toolbarConfig.operator.show.length;
			for ( ; i < len; i++ ){
				toolbarConfig.operator.show[i].isDisabled = "disable";
			}
			if ( toolbarConfig.operator.more ){
				toolbarConfig.operator.more.isDisabled = "disable";
			}
		}
	}
}])

pharosModule.factory("wirelessService", ["$rootScope", "$proxy", "$constant", "$url", "$translate", "$form", "$grid", "$timeout", function($rootScope, $proxy, $constant, $url, $translate, $form, $grid, $timeout) {
	var deviceUrl = $url.format("/device/");

	var wireless_mode_items = ["B", "G", "N", "BG", "BGN", "A", "N", "AN"];
	var wireless_mode_2_4g_items = [0, 1, 2, 3, 4];
	var wireless_mode_5g_items = [5, 6, 7];
	var channelWidthItems = {"5MHz":0, "10MHz":1, "20MHz":2, "20_40MHz":3};

	var wirelessTbl = [];
	var securityType;
	var securityEncrypt;
	var originDeviceName = "";

	var wirelessOptions = {
		proxy: {
			url: deviceUrl
		},
		fields: [{
			name: "deviceName",
			allowBlank: false
		},{
			name: "macAddress"
		},{
			name: "ipAddress"
		},{
			name: "product"
		},{
			name: "countryCode"
		},{
			name: "mode"
		},{
			name: "channelWidth"
		},{
			name: "mxTxRate"
		},{
			name: "channel"
		},{
			name: "switchTdma"
		},{
			name: "outputPower",
			vtype: {
				vtype: "float_number",
				config:{
					min: 0
				}
			},
			format: "integer"
		},{
			name: "distance",
			vtype: {
				vtype: "float_number",
				config: {
					max: 200,
					min: 0
				}
			},
			format: {
				format: "number",
				config: 1
			}
		},{
			name: "apIsolation"
		},{
			name: "autoDistance"
		}],
		autoLoad: false
	};

	function initWirelessEdit(scope, data, formObj/*, msgObj*/){
		var countryCode = data.wirelessCfg.countryCode;
		var wirelessType = data.wirelessCfg.wirelessType;
		// WIRELESS_MODE: ["Auto", "Access Point", "Client", "Repeater", "Bridge", "AP Router", "AP Client Router"]
		var wirelessMode = $constant.WIRELESS_MODE[data.wirelessMode+1];

		formObj.form.data.countryCode.name = $constant.COUNTRY[countryCode];
		// set global variable to remenber wireless info
		wirelessTbl = data.wirelessTbl;
		securityType = data.wirelessCfg.securityType;
		securityEncrypt = data.wirelessCfg.securityEncrypt;
		originDeviceName = data.deviceName;

		if(wirelessType){
			var current_mode_items, modeOptions = [];

			if(wirelessType == 1){//2.4g
				current_mode_items = wireless_mode_2_4g_items;
			}else if(wirelessType == 2){//5g
				current_mode_items = wireless_mode_5g_items;
			}
			for(var index = 0, len = current_mode_items.length; index < len; index++){
				modeOptions[index] = {};
				modeOptions[index].value = current_mode_items[index];
				modeOptions[index].name = $constant.MODE["MODE_"+wireless_mode_items[current_mode_items[index]]];
			}
			formObj.form.data.mode.options = modeOptions;
		}

		setOptions(formObj);

		var tips = $translate.instant("DEVICE.TX_POWER_RANGE");
		var maxTxPower = getMaxTxPower(formObj);
		maxTxPower !== -1 ? tips = tips.replace("%maxTxPower%", maxTxPower) : null;
		formObj.maxTxPowerTips = tips;
		formObj.form.data.outputPower.vtype.config.max = maxTxPower;

		formObj.distanceRangeTips = $translate.instant("DEVICE.DISTANCE_RANGE");

		// channel/frequecy can be choosed when the wireless mode is access point(0) or ap router mode(4)
		if(wirelessMode === "Access Point" || wirelessMode === "AP Router"){
			formObj.form.data.channel.disabled = false;
		}else{
			formObj.form.data.channel.disabled = true;
		}
		// ApIsolation can be choosed when the wireless mode is client
		if(wirelessMode === "Client"){
			formObj.form.data.apIsolation.disabled = true;
		}else{
			formObj.form.data.apIsolation.disabled = false;
		}
		// Access Point mode: MaxTream is checkbox
		// AP Router mode: can't set MaxTream
		// Client or AP Client Router, MaxTream is combo
		// other mode: MaxTream is invisible
		// MaxTream is checkbox at present
		if(wirelessMode === "Access Point"){
			formObj.form.data.switchTdma.disabled = false;
			formObj.form.data.switchTdma.visiable = true;
		}else if(wirelessMode === "AP Router"){
			formObj.form.data.switchTdma.disabled = true;
			formObj.form.data.switchTdma.visiable = true;
		}else if(wirelessMode === "Client" || wirelessMode === "AP Client Router"){
			formObj.form.data.switchTdma.disabled = false;
			formObj.form.data.switchTdma.visiable = true;
		}else{
			formObj.form.data.switchTdma.disabled = true;
			formObj.form.data.switchTdma.visiable = false;
		}
	}

	function setOptions(formObj){
		var flag5MHz = false, flag20_40MHz = false;
		var mode = formObj.form.data.mode.data;
		var currentChannelWidth = angular.copy(channelWidthItems);
		var channelWidthOptions = [];
		var itemIndex = 0;

		// if(mode){
			if(formObj.form.data.countryCode.data == 511){
				// only testmode support 5MHz at present
				flag5MHz = true;
			}
			if(wireless_mode_items[mode] && wireless_mode_items[mode].indexOf("N") !== -1){
				for(var index = 0, len = wirelessTbl.length; index < len; index++){
					if(wirelessTbl[index].channelWidthFlag & 6){
						flag20_40MHz = true;
						break;
					}
				}
			}

			if(flag5MHz == false){
				delete currentChannelWidth["5MHz"];
			}
			if(flag20_40MHz == false){
				delete currentChannelWidth["20_40MHz"];
			}
			for(var item in currentChannelWidth){
				channelWidthOptions[itemIndex] = {};
				channelWidthOptions[itemIndex].value = currentChannelWidth[item];
				channelWidthOptions[itemIndex++].name = "DEVICE."+ item.toUpperCase();
			}
			formObj.form.data.channelWidth.options = channelWidthOptions;
			formObj.form.data.channelWidth.data = channelWidthOptions[channelWidthOptions.length -1].value;

			setChannel(formObj);
			setMxTxRate(formObj);
		// }
	}

	function getMaxTxPower(formObj){
		var len = wirelessTbl.length;
		if(wirelessTbl && len !== 0){
			// var channelIndex = getIndex($scope.wireless.form.data.channel.data, $scope.wireless.form.data.channel.options);
			var channelIndex = formObj.form.data.channel.data;
			if(channelIndex == 0){
				var maxTxPower = -1;
				for(var index = 0; index < len; index++){
					if(maxTxPower < wirelessTbl[index].maxTxPower){
						maxTxPower = wirelessTbl[index].maxTxPower;
					}
				}
				return maxTxPower;
			}else if(channelIndex !== -1 && len >= channelIndex){
				return wirelessTbl[channelIndex-1].maxTxPower;
			}
		}else{
			return -1;
		}
	}

	function setChannel(formObj){
		var channelWidth = formObj.form.data.channelWidth.data;
		var channelOptions = [];
		channelOptions.push({value: 0, name: "DEVICE.AUTO"});
		if(wirelessTbl){
			for(var index = 0, len = wirelessTbl.length; index < len; index++){
				var channelOption = {};
				if(channelWidth == channelWidthItems["20MHz"] && (wirelessTbl[index].channelWidthFlag & 2 == 0)){
					continue;
				}else if(channelWidth == channelWidthItems["20_40MHz"] && (wirelessTbl[index].channelWidthFlag & 6 == 0)){
					continue;
				}
				var channelValue = mhz2Channel(wirelessTbl[index].frequency);
				channelOption.value = index + 1;
				channelOption.name = channelValue + " / " + wirelessTbl[index].frequency + "MHz";  //���޸ģ��ַ�����Դ
				channelOptions.push(channelOption);
			}
			formObj.form.data.channel.options = channelOptions;
		}
	}

	function mhz2Channel(freq){
		freq = angular.isString(freq) ? parseInt(freq) : freq;
		if(freq == 2484){
			return 14;
		}
		if(freq <= 2402){
			return 237 + (freq - 2312) / 5;
		}else if(freq < 2484){
			return (freq - 2407) / 5;
		}else if(freq <= 2507){
			return 76 + (freq - 2487) / 5;
		}else if(freq < 4000){
			return 15 + (freq - 2512) / 5;
		}else if(freq <= 5000){
			return (freq - 4000) / 5;
		}else{
			return (freq - 5000) / 5;
		}
	}

	function setMxTxRate(formObj){
		var mode = formObj.form.data.mode.data;
		var channelWidth = formObj.form.data.channelWidth.data;
		var currentMxTxRateItems = [];
		var mxTxRateOptions = [];
		var Index2Mode = 0;

		var nonsequence = false;
		function handleUnSeq(index) {
		    // [1, 8] & [13, 16]
		    if (index >= 0 && index <= 7) {
		        return index + 1;
		    } else if (index >= 8 && index <= 11) {
		        return index + 5;
		    } else {
		        return -1;
		    }
		}

		if(mode >= 0 && mode !== ""){
			if("B" == wireless_mode_items[mode]){
				currentMxTxRateItems = $constant.MXTXRATE["B"][channelWidth];
				Index2Mode = 1;
			}else if("G" == wireless_mode_items[mode] ||
					 "BG" == wireless_mode_items[mode] ||
					 "A" == wireless_mode_items[mode]){

				currentMxTxRateItems = $constant.MXTXRATE["A_BG_G"][channelWidth];
				Index2Mode = 1;
			}else{
				// securityType, 0:none, 1:wep, 2:wpa, 3:psk
				// securityEncrypt, 1:auto, 2:tkip, 3:aes
				if(securityType == 1 || securityEncrypt == 2){
					currentMxTxRateItems = $constant.MXTXRATE["N_TKIP"][channelWidth];
					Index2Mode = 1;
				}else{
					currentMxTxRateItems = $constant.MXTXRATE["N"][channelWidth];
					nonsequence = true;
				}
			}
			for(var index = 0, len = currentMxTxRateItems.length; index < len; index++){
				mxTxRateOptions[index] = {};
				mxTxRateOptions[index].value = index + Index2Mode;
				if ( nonsequence ){
					mxTxRateOptions[index].value = handleUnSeq(index);
				}

				mxTxRateOptions[index].name = currentMxTxRateItems[index];
			}
			formObj.form.data.mxTxRate.options= mxTxRateOptions;
			formObj.form.data.mxTxRate.data = mxTxRateOptions[mxTxRateOptions.length -1].value;
		}
	}

	return {
		initWirelessMsg: function( scope, msgObj, formObj ){
			angular.extend( msgObj, {
				title: "DEVICE.EDITDEVICE",
				okBtn: {
					handler: function(){}
				},
				cancelBtn: {
					handler: function(){}
				},
				autoClose: false
			})
		},
		configWirelessForm: function( formObj ){
			$form.config(formObj, wirelessOptions);
		},
		getOriginDeviceName: function(){
			return originDeviceName;
		},
		editWireless: function(scope, formObj, msgObj){
			// get info
			var selectedIndexs = scope.grid.selectedIndexs;
			var editDeviceID, formData;
			if(scope.grid.selectedIndexs.length !== 0){
				editDeviceID = scope.grid.data[selectedIndexs[0]].key;
				$proxy.request({
					url: deviceUrl
				},{
					"REQUEST": "GET_DEVICE_WIRELESS_CFG",
					"DATA": {
						"deviceID": editDeviceID
					}
				}, function(data){
					// init form
					if(data){
						formData =  {deviceName: data.deviceName};
						if(data.wirelessCfg && data.wirelessCfg.deviceID === editDeviceID){
							angular.extend(formData, data.wirelessCfg);
							$form.loadData(formObj, formData);
							initWirelessEdit(scope, data, formObj);
							msgObj["show"] = true;
							$timeout(function(){
								$form.loadData(formObj, formData);
								$timeout(function(){
									formObj.form.data.mxTxRate.data = formData.mxTxRate;
								}, 10);
							}, 150);
						}
					}
				}, function(){}, function(){});
			}
		},
		initChangeHandler: function(formObj){
			formObj.form.data.mode.changeHandler = function(nVal){
				if(nVal || nVal === 0){
					setOptions(formObj);
				}
			};

			formObj.form.data.channelWidth.changeHandler = function(nVal, oVal){
				var existedFlag = false;
				var channel = formObj.form.data.channel.data;
				if(nVal !== undefined){
					setChannel(formObj);
					setMxTxRate(formObj);
					// if channel not existed, set data to auto
					for(var index = 0, len = formObj.form.data.channel.options.length; index < len; index++){
						if(formObj.form.data.channel.options[index].value === channel){
							existedFlag = true;
							break;
						}
					}
					formObj.form.data.channel.data = existedFlag ? channel : "auto";
				}
			}

			formObj.form.data.autoDistance.changeHandler = function(nVal, oVal){
				if(nVal === 0){
					formObj.form.data.distance.disabled = false;
				}else if(nVal === 1){
					formObj.form.data.distance.disabled = true;
				}
			}
		}

	}

}]);
